<?php
//   Tool Functions for Image List Creation
//
// Jun Kuwamura <juk@yokohama.email.ne.jp>
//	Mon Jan 22 01:22:57 JST 2001
//
// 2002-09-16 JuK Add extra info from server in remarks.
// 2002-09-15 JuK Seperate utility and file check functions to another file.
// 2002-08-28 JuK Add (Re)Create image for zero sized file in black.
// 2002-07-04 JuK Mod ReCreate for original sized image.
// 2002-07-04 JuK Add image modtime function.
// 2002-02-17 JuK devide tool functions to here.
//
require_once("imagekanji.php");
require_once("checkutil.php");

///////////////////////////////////////
// Check GD module and load it if not
///////////////////////////////////////
if(!extension_loaded('gd')){
  if(!dl('gd.so')){
    echo 'error';
    exit;
  }
}

///////////////////////////////////////////////////////////////////////
// Get image format type
//	$img_path: image file path
///////////////////////////////////////////////////////////////////////
function GetImgType( $img_path )
{
  if ( ! file_exists("$img_path") ) {
    Debug("Error:  File is not existed:  $img_path", 9);
    return 0;
  }
  if ( is_dir("$img_path") ) {
    Debug("Error:  File is a directory:  $img_path", 9);
    return 0;
  }
  $iminfo = array();
  if ( filesize($img_path) == 0 ) {
    $iminfo[0]=0;
    $iminfo[1]=0;
    $iminfo[2]=0;
  } else {
    $iminfo = GetImageSize("$img_path");
  }
  // $iminfo[2]: image format id(1=GIF, 2=JPG, 3=PNG, 4=SWF)
  if ( $iminfo[2] != 2 && $iminfo[2] != 3 && $iminfo[2] !=4 ) {	//JPG|PNG|SWF
    Debug("Error:  File is not in JPEG|PNG|SWF format:  $img_path", 9);
    return 0;
  }
  return $iminfo[2];
}

///////////////////////////////////////////////////////////////////////
// Get image pixels in "{width}x{height}" format
//	$img_path: image file path
///////////////////////////////////////////////////////////////////////
function GetImgPixels( $img_path )
{
  $iminfo = array();
  if ( file_exists($img_path) ) {
    if ( filesize($img_path) == 0 ) {
      $iminfo[0]=0;
      $iminfo[1]=0;
      $iminfo[2]=0;
    } else {
      $iminfo=GetImageSize("$img_path");
    }
  } else {
    Error("File is not existed: $img_path", __LINE__);
    return;
  }
  Debug("File:$img_path type = $iminfo[2]", 5);
  // $iminfo[2]: image format id(1=GIF, 2=JPG, 3=PNG, 4=SWF)
  if ( $iminfo[2] != 2 && $iminfo[2] != 3 && $iminfo[2] !=4 ) {	//JPG|PNG|SWF
    Debug("Error:  File is not in JPEG|PNG|SWF format:  $img_path", 9);
  }
  $img_pixels=$iminfo[0]."x".$iminfo[1];
  Debug("pixcels=$img_pixels", 9);
  return $img_pixels;
}

///////////////////////////////////////////////////////////////////////
// Get image modification time
//	$img_path: image file path
///////////////////////////////////////////////////////////////////////
function GetImgModTime( $img_path )
{
    return strftime("%Y-%m-%d %H:%M", filemtime("$img_path"));
}

///////////////////////////////////////////////////////////////////////
// Create a image from reading a image file in
//	$img_file: image file path name to read in
///////////////////////////////////////////////////////////////////////
function CreatImgFrom( $img_file )
{
  switch ( GetImgType($img_file) ) {
  case 2:	// JPG
    $img = ImageCreateFromJpeg($img_file);
    break;
  case 3:	// PNG
    $img = ImageCreateFromPNG($img_file);
    break;
  case 4:	// SWF
    $img = ImageCreateFromWBMP($img_file);
    break;
  default:
    return;
    break;
  }
  return $img;
}

///////////////////////////////////////////////////////////////////////
// Write a image file out
//	$img: internal image
//	$img_file: image file path name to write out
//	$img_fmt: image format(jpeg|png|wbmp)
//	$qty: image quolity for JPEG compression(0-100)
///////////////////////////////////////////////////////////////////////
function SaveImgTo( $img, $img_file, $img_fmt="jpeg", $qty="81" )
{
  global $TEST;
  if ( $TEST > 3 ) {
    include( "print_func_args.php" );
  }

  // ImageInterlace($img, 1);

  switch ( strtolower($img_fmt) ) {
  case "jpeg":	// JPG
    ImageJpeg($img, $img_file, $qty);
    break;
  case "png":	// PNG
    ImagePNG($img, $img_file);
    break;
  case "wbmp":	// SWF
    ImageWBMP($img, $img_file);
    break;
  default:
    return;
    break;
  }
}

///////////////////////////////////////////////////////////////////////
// Read remarks for image from file.
//	$dir: directory
//	$img: image name(not a actual remark file name) 
///////////////////////////////////////////////////////////////////////
function RdImgRemark( $dir, $img, $line = __LINE__ )
{
  global $TEST;
  if ( $TEST > 3 ) {
    echo "Line: $line";
    include( "print_func_args.php" );
  }

  // The remarks file extention to be ".txt".
  $pieces = explode(".", $img);
  $file = "$pieces[0].txt";

  // Check readable and read remarks.
  $filename = ChkReadable($dir, $file, __LINE__);
  if ( empty($filename) ) {
    return;
  } else {
    $fd = fopen ($filename, "r");
    if ($fd) {
      $rmk = fread($fd, filesize($filename));
    } else {
      Error("$filename could not be opened to read.", $line);
      return;
    }
    fclose ($fd);
  }
  return $rmk;
}

///////////////////////////////////////////////////////////////////////
// Write remarks for image from file.
//	$dir: directory
//	$img: image name(not a remark file name) 
//	$rmk: remarks to save in a file
//	$edit: o: overwrite, a: append
///////////////////////////////////////////////////////////////////////
function WtImgRemark( $dir, $img, $rmk, $line = __LINE__, $edit="o" )
{
  global $TEST;
  if ( $TEST > 3 ) {
    echo "Line: $line";
    include( "print_func_args.php" );
  }
  $browser=$_SERVER['HTTP_USER_AGENT'];
  $clihost=$_SERVER['HTTP_HOST'];
  $curdate=date("D M j G:i:s T Y");

  // The remarks file extention to be ".txt".
  $pieces = explode(".", $img);
  $file = "$pieces[0].txt";

  // Check writable and write remarks.
  $filename = ChkWritable($dir, $file, __LINE__);
  if ( empty($filename) ) {
    return 0;
  } else {
    if (! file_exists($filename) || filesize($filename) == 0 || $edit != "a") {
      $fd = fopen ($filename, "w");
      $edit = "o";
    } else {
      $fd = fopen ($filename, "r+");
    }
    if ($fd) {
      if ( $edit == "a" ) {
	fseek( $fd, 0, SEEK_END );
	$ret = fwrite($fd, "\n--\t--\n");
	if ($ret == -1 ) {
	  Error("write into the file: $filename.", $line);
	  return 0;
	}
      }
      $ret = fwrite($fd, "\t__ $curdate __\t\n");
      $ret = fwrite($fd, $rmk);
      $ret = fwrite($fd, "\n\t__ $browser @ $clihost __\t");
    } else {
      Error("$filename could not be opened to write.", $line);
      return 0;
    }
    fclose ($fd);
  }
  return $ret;
}

////////////////////////////////////////////////////////////////////////
// Image(Icon) re-creation with some infomation text
//	$new_w: output image width in pixels.
//	$new_h: output image height in pixels.
//	$img_file: image file name.
//	$new_dir: directory for output image.
//	$org_dir: directory for output image.
//	$img_fmt: output image format(jpeg|png|wbmp)
////////////////////////////////////////////////////////////////////////
function ReCreatImg( $new_w, $new_h, $img_file, $new_dir = ".", $org_dir = ".", $remark = "", $img_fmt = "jpeg" )
{
  Debug("ReCreatImg( $new_w, $new_h, $img_file, $new_dir, $org_dir, $remark, $img_fmt)", 1);

  $filesize = filesize("$org_dir/$img_file");
  $modtime =  GetImgModTime( "$org_dir/$img_file" );

  // Check the directory and the file for a new image.
  $filename = ChkWritable("$new_dir", "$img_file", __LINE__);
  if ( empty($filename) ) {
    return 0;
  }
  // Check Source Image
  $filename = ChkReadable("$org_dir", "$img_file", __LINE__);
  $imagepixels = GetImgPixels($filename);
  if ( ! $imagepixels ) {
    Error("Original image file error:  $org_dir/$img_file");
    return 0;
  }

  // Set and calculate size informations for a new image
  $font_id = 1;
  if( extension_loaded('gd') ){
    $fw=ImageFontWidth($font_id);
    $fh=ImageFontHeight($font_id);
  } else {
    $fw = 6;
    $fh = 6;
  }
  Debug("font height=$fw", 9);

  Debug ("<A href=$org_dir/$img_file>", 9); 
  Debug ("$org_dir/$img_file", 9); 
  if ( $filesize > 0 ) {
    // Read original image and set valiables for resize
    $org_img = CreatImgFrom("$org_dir/$img_file"); 
    $org_w = ImageSX($org_img); 
    $org_h = ImageSY($org_img); 
  } else {
    $org_img = ImageCreate ($new_h, $new_w);
    $background_color = ImageColorAllocate ($org_img, 0, 0, 0);
    $org_w = $new_w;
    $org_h = $new_h;
  }
  if ( $new_w != 0 || $new_h != 0 ) {
    // re-calc. new_? variables
    $base_w=$new_w+$fh*2;
    $base_h=$new_h+$fh*2;
    $fact = Min($new_w/$org_w, $new_h/$org_h);
    $new_w = $org_w*$fact;
    $new_h = $org_h*$fact;
    $new_x = ($base_w - $new_w)/2;
    $new_y = ($base_h - $new_h)/2;

    // Create new image canvas
    $new_img = ImageCreate($base_w, $base_h);
    // set bgcolor
    $bg_color = ImageColorAllocate($new_img, 255, 255, 255);
    // Format a new image base canvas and write informations on it.
    ImageFill($new_img, 0, 0, $bg_color);
    // Resize and copy the original image.
    ImageCopyResized($new_img, $org_img, $new_x, $new_y, 0, 0, $new_w, $new_h, $org_w, $org_h);
    ImageDestroy($org_img);
    $new_file=$img_file;
  } else {
    $font_id = 2;
    if( extension_loaded('gd') ){
      $fw=ImageFontWidth($font_id);
      $fh=ImageFontHeight($font_id);
    } else {
      $fw=8;
      $fh=10;
    }
    $new_img = $org_img;
    $base_w=$org_w;
    $base_h=$org_h;
    $new_x=$fh;
    $new_y=$fh;
    $new_file="org.$img_file";
  }

  //
  // write strings to the image
  //
  // set string colors
  $text_color = ImageColorAllocate($new_img, 233, 14, 91);
  $text_color2 = ImageColorAllocate($new_img, 14, 233, 91);

  //  write file name at the top left
  $y = strlen("$img_file") * $fw;
  ImageStringUp($new_img, $font_id, 0, $y,  "$img_file", $text_color);

  //  write original modification date at the top right
  $x = $base_w - (strlen("$modtime") * $fw);
  ImageString($new_img, $font_id, $x, 0,  "$modtime", $text_color);

  //  write original file size and pixels at the bottom
  $y = $base_h - $fh;
  ImageString($new_img, $font_id, 0, $y,  "($imagepixels, $filesize)", $text_color);

  //  write remark. 
  $x = $base_w - ($new_x*1.5);
  $y = $base_h - ($new_y*1.5);
  ImageKanjiUp($new_img, ($font_id+1), $x, $y,  $remark, $text_color);

  // Write a new image file in JPEG format.
  SaveImgTo($new_img, "$new_dir/$new_file");
  
  ImageDestroy($new_img);
  return 1;
}
?>
